(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@visactor/vtable'), require('vue'), require('@visactor/vutils')) :
  typeof define === 'function' && define.amd ? define(['exports', '@visactor/vtable', 'vue', '@visactor/vutils'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VueVTable = {}, global.VTable, global.vue, global.vutils));
})(this, (function (exports, VTable, vue, vutils) { 'use strict';

  function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
      Object.keys(e).forEach(function (k) {
        if (k !== 'default') {
          var d = Object.getOwnPropertyDescriptor(e, k);
          Object.defineProperty(n, k, d.get ? d : {
            enumerable: true,
            get: function () { return e[k]; }
          });
        }
      });
    }
    n.default = e;
    return Object.freeze(n);
  }

  var VTable__namespace = /*#__PURE__*/_interopNamespaceDefault(VTable);

  function toCamelCase(str) {
      return str.replace(/-([a-z])/g, g => g[1].toUpperCase());
  }
  function convertPropsToCamelCase(props) {
      const newProps = {};
      for (const key in props) {
          if (props.hasOwnProperty(key)) {
              const camelCaseKey = toCamelCase(key);
              newProps[camelCaseKey] = props[key];
          }
      }
      return newProps;
  }

  function flattenVNodes(vnodes) {
      return vnodes.flatMap(vnode => (Array.isArray(vnode.children) ? flattenVNodes(vnode.children) : vnode));
  }

  function isEventProp(key, props) {
      return key.startsWith('on') && vutils.isFunction(props[key]);
  }
  function createCustomLayout(children) {
      const componentMap = {
          Group: VTable__namespace.CustomLayout.Group,
          Image: VTable__namespace.CustomLayout.Image,
          Text: VTable__namespace.CustomLayout.Text,
          Tag: VTable__namespace.CustomLayout.Tag,
          Radio: VTable__namespace.CustomLayout.Radio,
          CheckBox: VTable__namespace.CustomLayout.CheckBox
      };
      function createComponent(child) {
          if (!child) {
              return null;
          }
          const { type, children: childChildren } = child;
          const props = convertPropsToCamelCase(child.props);
          const componentName = type?.symbol || type?.name;
          const ComponentClass = componentMap[componentName];
          if (!ComponentClass) {
              return null;
          }
          const component = new ComponentClass({ ...props });
          bindComponentEvents(component, props);
          const subChildren = resolveChildren(childChildren);
          subChildren.forEach((subChild) => {
              const subComponent = createComponent(subChild);
              if (subComponent) {
                  component.add(subComponent);
              }
              else if (subChild.type === Symbol.for('v-fgt')) {
                  subChild.children.forEach((nestedChild) => {
                      const nestedComponent = createComponent(nestedChild);
                      if (nestedComponent) {
                          component.add(nestedComponent);
                      }
                  });
              }
          });
          return component;
      }
      function resolveChildren(childChildren) {
          return childChildren?.default?.() || childChildren || [];
      }
      function bindComponentEvents(component, props) {
          Object.keys(props).forEach(key => {
              if (isEventProp(key, props)) {
                  let eventName;
                  if (key.startsWith('on')) {
                      eventName = key.slice(2).toLowerCase();
                  }
                  else {
                      eventName = toCamelCase(key.slice(2)).toLowerCase();
                  }
                  component.addEventListener(eventName, props[key]);
              }
          });
      }
      return { rootComponent: createComponent(children) };
  }
  function createCustomLayoutHandler(children) {
      return (args) => {
          const { table, row, col, rect } = args;
          const record = table.getCellOriginRecord(col, row);
          const { height, width } = rect ?? table.getCellRect(col, row);
          const rootContainer = children.customLayout({ table, row, col, rect, record, height, width })[0];
          const { rootComponent } = createCustomLayout(rootContainer);
          return {
              rootContainer: rootComponent,
              renderDefault: false
          };
      };
  }

  function extractPivotSlotOptions(vnodes) {
      const options = {
          columns: [],
          columnHeaderTitle: [],
          rows: [],
          rowHeaderTitle: [],
          indicators: [],
          corner: {},
          tooltip: {},
          menu: {}
      };
      const typeMapping = {
          PivotColumnDimension: 'columns',
          PivotColumnHeaderTitle: 'columnHeaderTitle',
          PivotRowDimension: 'rows',
          PivotRowHeaderTitle: 'rowHeaderTitle',
          PivotCorner: 'corner',
          PivotIndicator: 'indicators',
          Tooltip: 'tooltip',
          Menu: 'menu'
      };
      vnodes.forEach(vnode => {
          vnode.props = convertPropsToCamelCase(vnode.props);
          const typeName = vnode.type?.symbol || vnode.type?.name;
          const optionKey = typeMapping[typeName];
          if (optionKey) {
              if (Array.isArray(options[optionKey])) {
                  if (vnode.props.hasOwnProperty('objectHandler')) {
                      options[optionKey].push(vnode.props.objectHandler);
                  }
                  else {
                      options[optionKey].push(vnode.props);
                  }
              }
              else {
                  options[optionKey] = vnode.props;
              }
          }
      });
      return options;
  }
  function extractListSlotOptions(vnodes) {
      const options = {
          columns: [],
          tooltip: {},
          menu: {}
      };
      const typeMapping = {
          ListColumn: 'columns',
          Tooltip: 'tooltip',
          Menu: 'menu'
      };
      vnodes.forEach(vnode => {
          vnode.props = convertPropsToCamelCase(vnode.props);
          const typeName = vnode.type?.symbol || vnode.type?.name;
          const optionKey = typeMapping[typeName];
          if (optionKey) {
              if (optionKey === 'columns' && vnode.children) {
                  vnode.props.customLayout = createCustomLayoutHandler(vnode.children);
              }
              if (Array.isArray(options[optionKey])) {
                  options[optionKey].push(vnode.props);
              }
              else {
                  options[optionKey] = vnode.props;
              }
          }
      });
      return options;
  }
  function mergeSlotOptions(propsOptions, slotOptions) {
      return {
          ...propsOptions,
          columns: slotOptions.columns && slotOptions.columns.length ? slotOptions.columns : propsOptions.columns,
          columnHeaderTitle: slotOptions.columnHeaderTitle && slotOptions.columnHeaderTitle.length
              ? slotOptions.columnHeaderTitle
              : propsOptions.columnHeaderTitle,
          rows: slotOptions.rows && slotOptions.rows.length ? slotOptions.rows : propsOptions.rows,
          rowHeaderTitle: slotOptions.rowHeaderTitle && slotOptions.rowHeaderTitle.length
              ? slotOptions.rowHeaderTitle
              : propsOptions.rowHeaderTitle,
          indicators: slotOptions.indicators && slotOptions.indicators.length ? slotOptions.indicators : propsOptions.indicators,
          corner: Object.keys(propsOptions.corner || {}).length ? propsOptions.corner : slotOptions.corner,
          tooltip: Object.keys(slotOptions.tooltip || {}).length ? slotOptions.tooltip : propsOptions.tooltip,
          menu: Object.keys(slotOptions.menu || {}).length ? slotOptions.menu : propsOptions.menu
      };
  }

  const EVENT_TYPE = {
      ...VTable.ListTable.EVENT_TYPE,
      ...VTable.PivotTable.EVENT_TYPE,
      ...VTable.PivotChart.EVENT_TYPE
  };
  const TABLE_EVENTS = {
      onClickCell: EVENT_TYPE.CLICK_CELL,
      onDblClickCell: EVENT_TYPE.DBLCLICK_CELL,
      onMouseDownCell: EVENT_TYPE.MOUSEDOWN_CELL,
      onMouseUpCell: EVENT_TYPE.MOUSEUP_CELL,
      onSelectedCell: EVENT_TYPE.SELECTED_CELL,
      onKeyDown: EVENT_TYPE.KEYDOWN,
      onMouseEnterTable: EVENT_TYPE.MOUSEENTER_TABLE,
      onMouseLeaveTable: EVENT_TYPE.MOUSELEAVE_TABLE,
      onMouseDownTable: EVENT_TYPE.MOUSEDOWN_TABLE,
      onMouseMoveCell: EVENT_TYPE.MOUSEMOVE_CELL,
      onMouseEnterCell: EVENT_TYPE.MOUSEENTER_CELL,
      onMouseLeaveCell: EVENT_TYPE.MOUSELEAVE_CELL,
      onContextMenuCell: EVENT_TYPE.CONTEXTMENU_CELL,
      onResizeColumn: EVENT_TYPE.RESIZE_COLUMN,
      onResizeColumnEnd: EVENT_TYPE.RESIZE_COLUMN_END,
      onChangeHeaderPosition: EVENT_TYPE.CHANGE_HEADER_POSITION,
      onChangeHeaderPositionStart: EVENT_TYPE.CHANGE_HEADER_POSITION_START,
      onChangeHeaderPositionFail: EVENT_TYPE.CHANGE_HEADER_POSITION_FAIL,
      onSortClick: EVENT_TYPE.SORT_CLICK,
      onFreezeClick: EVENT_TYPE.FREEZE_CLICK,
      onScroll: EVENT_TYPE.SCROLL,
      onDropdownMenuClick: EVENT_TYPE.DROPDOWN_MENU_CLICK,
      onMouseOverChartSymbol: EVENT_TYPE.MOUSEOVER_CHART_SYMBOL,
      onDragSelectEnd: EVENT_TYPE.DRAG_SELECT_END,
      onDropdownIconClick: EVENT_TYPE.DROPDOWN_ICON_CLICK,
      onDropdownMenuClear: EVENT_TYPE.DROPDOWN_MENU_CLEAR,
      onTreeHierarchyStateChange: EVENT_TYPE.TREE_HIERARCHY_STATE_CHANGE,
      onShowMenu: EVENT_TYPE.SHOW_MENU,
      onHideMenu: EVENT_TYPE.HIDE_MENU,
      onIconClick: EVENT_TYPE.ICON_CLICK,
      onLegendItemClick: EVENT_TYPE.LEGEND_ITEM_CLICK,
      onLegendItemHover: EVENT_TYPE.LEGEND_ITEM_HOVER,
      onLegendItemUnHover: EVENT_TYPE.LEGEND_ITEM_UNHOVER,
      onLegendChange: EVENT_TYPE.LEGEND_CHANGE,
      onMouseEnterAxis: EVENT_TYPE.MOUSEENTER_AXIS,
      onMouseLeaveAxis: EVENT_TYPE.MOUSELEAVE_AXIS,
      onCheckboxStateChange: EVENT_TYPE.CHECKBOX_STATE_CHANGE,
      onRadioStateChange: EVENT_TYPE.RADIO_STATE_CHANGE,
      onAfterRender: EVENT_TYPE.AFTER_RENDER,
      onInitialized: EVENT_TYPE.INITIALIZED,
      onPivotSortClick: EVENT_TYPE.PIVOT_SORT_CLICK,
      onDrillMenuClick: EVENT_TYPE.DRILLMENU_CLICK,
      onVChartEventType: EVENT_TYPE.VCHART_EVENT_TYPE,
      onChangeCellValue: EVENT_TYPE.CHANGE_CELL_VALUE,
      onMousedownFillHandle: EVENT_TYPE.MOUSEDOWN_FILL_HANDLE,
      onDragFillHandleEnd: EVENT_TYPE.DRAG_FILL_HANDLE_END,
      onDblclickFillHandle: EVENT_TYPE.DBLCLICK_FILL_HANDLE,
      onScrollVerticalEnd: EVENT_TYPE.SCROLL_VERTICAL_END,
      onScrollHorizontalEnd: EVENT_TYPE.SCROLL_HORIZONTAL_END,
      onChangCellValue: EVENT_TYPE.CHANGE_CELL_VALUE
  };
  const TABLE_EVENTS_KEYS = Object.keys(TABLE_EVENTS);

  var _sfc_main$3 = vue.defineComponent({
      __name: 'base-table',
      props: {
          type: { type: String, required: false },
          options: { type: null, required: false },
          records: { type: Array, required: false },
          width: { type: [Number, String], required: false, default: '100%' },
          height: { type: [Number, String], required: false, default: '100%' },
          onReady: { type: Function, required: false },
          onError: { type: Function, required: false },
          onClickCell: { type: null, required: false },
          onDblClickCell: { type: null, required: false },
          onMouseDownCell: { type: null, required: false },
          onMouseUpCell: { type: null, required: false },
          onSelectedCell: { type: null, required: false },
          onKeyDown: { type: null, required: false },
          onMouseEnterTable: { type: null, required: false },
          onMouseLeaveTable: { type: null, required: false },
          onMouseDownTable: { type: null, required: false },
          onMouseMoveCell: { type: null, required: false },
          onMouseEnterCell: { type: null, required: false },
          onMouseLeaveCell: { type: null, required: false },
          onContextMenuCell: { type: null, required: false },
          onResizeColumn: { type: null, required: false },
          onResizeColumnEnd: { type: null, required: false },
          onChangeHeaderPosition: { type: null, required: false },
          onChangeHeaderPositionStart: { type: null, required: false },
          onChangeHeaderPositionFail: { type: null, required: false },
          onSortClick: { type: null, required: false },
          onFreezeClick: { type: null, required: false },
          onScroll: { type: null, required: false },
          onDropdownMenuClick: { type: null, required: false },
          onMouseOverChartSymbol: { type: null, required: false },
          onDragSelectEnd: { type: null, required: false },
          onDropdownIconClick: { type: null, required: false },
          onDropdownMenuClear: { type: null, required: false },
          onTreeHierarchyStateChange: { type: null, required: false },
          onShowMenu: { type: null, required: false },
          onHideMenu: { type: null, required: false },
          onIconClick: { type: null, required: false },
          onLegendItemClick: { type: null, required: false },
          onLegendItemHover: { type: null, required: false },
          onLegendItemUnHover: { type: null, required: false },
          onLegendChange: { type: null, required: false },
          onMouseEnterAxis: { type: null, required: false },
          onMouseLeaveAxis: { type: null, required: false },
          onCheckboxStateChange: { type: null, required: false },
          onRadioStateChange: { type: null, required: false },
          onAfterRender: { type: null, required: false },
          onInitialized: { type: null, required: false },
          onPivotSortClick: { type: null, required: false },
          onDrillMenuClick: { type: null, required: false },
          onVChartEventType: { type: null, required: false },
          onChangeCellValue: { type: null, required: false },
          onMousedownFillHandle: { type: null, required: false },
          onDragFillHandleEnd: { type: null, required: false },
          onDblclickFillHandle: { type: null, required: false },
          onScrollVerticalEnd: { type: null, required: false },
          onScrollHorizontalEnd: { type: null, required: false }
      },
      emits: TABLE_EVENTS_KEYS,
      setup(__props, { expose: __expose, emit: __emit }) {
          const props = __props;
          const vTableContainer = vue.ref(null);
          const vTableInstance = vue.shallowRef(null);
          __expose({ vTableInstance });
          const containerWidth = vue.computed(() => (typeof props.width === 'number' ? `${props.width}px` : props.width));
          const containerHeight = vue.computed(() => (typeof props.height === 'number' ? `${props.height}px` : props.height));
          const emit = __emit;
          const bindEvents = (instance) => {
              TABLE_EVENTS_KEYS.forEach(eventKey => {
                  const vueEventHandler = (event) => {
                      emit(eventKey, event);
                  };
                  instance.on(TABLE_EVENTS[eventKey], vueEventHandler);
              });
          };
          const createTableInstance = (Type, options) => {
              vTableInstance.value = new Type(vTableContainer.value, options);
          };
          const createVTable = () => {
              if (!vTableContainer.value) {
                  return;
              }
              if (vTableInstance.value) {
                  vTableInstance.value.release();
              }
              const getRecords = () => {
                  return props.records !== undefined && props.records !== null && props.records.length > 0
                      ? props.records
                      : props.options.records;
              };
              try {
                  switch (props.type) {
                      case 'list':
                          createTableInstance(VTable.ListTable, {
                              ...props.options,
                              records: getRecords()
                          });
                          break;
                      case 'pivot':
                          createTableInstance(VTable.PivotTable, {
                              ...props.options,
                              records: getRecords()
                          });
                          break;
                      case 'chart':
                          createTableInstance(VTable.PivotChart, {
                              ...props.options,
                              records: getRecords()
                          });
                          break;
                  }
                  bindEvents(vTableInstance.value);
                  props.onReady?.(vTableInstance.value, true);
              }
              catch (err) {
                  props.onError?.(err);
              }
          };
          const updateVTable = (newOptions) => {
              if (!vTableInstance.value) {
                  return;
              }
              try {
                  switch (props.type) {
                      case 'list':
                          if (vTableInstance.value instanceof VTable.ListTable) {
                              vTableInstance.value.updateOption(newOptions);
                          }
                          break;
                      case 'pivot':
                          if (vTableInstance.value instanceof VTable.PivotTable) {
                              vTableInstance.value.updateOption(newOptions);
                          }
                          break;
                      case 'chart':
                          if (vTableInstance.value instanceof VTable.PivotChart) {
                              vTableInstance.value.updateOption(newOptions);
                          }
                          break;
                  }
              }
              catch (err) {
                  props.onError?.(err);
              }
          };
          vue.onMounted(createVTable);
          vue.onBeforeUnmount(() => vTableInstance.value?.release());
          vue.watch(() => props.options, newOptions => {
              if (vTableInstance.value) {
                  updateVTable(newOptions);
              }
              else {
                  createVTable();
              }
          });
          vue.watch(() => props.records, (newRecords, oldRecords) => {
              if (vTableInstance.value) {
                  updateVTable({ ...props.options, records: newRecords });
              }
              else {
                  createVTable();
              }
          }, { deep: true });
          return (_ctx, _cache) => {
              return (vue.openBlock(), vue.createElementBlock("div", {
                  ref_key: "vTableContainer",
                  ref: vTableContainer,
                  style: vue.normalizeStyle([{ width: containerWidth.value, height: containerHeight.value }, { "position": "relative" }])
              }, null, 4));
          };
      }
  });

  var _sfc_main$2 = vue.defineComponent({
      __name: 'list-table',
      props: {
          options: { type: Object, required: true },
          records: { type: Array, required: false },
          width: { type: [String, Number], required: false },
          height: { type: [String, Number], required: false }
      },
      setup(__props, { expose: __expose }) {
          const props = __props;
          const baseTableRef = vue.ref(null);
          const slots = vue.useSlots();
          const computedOptions = vue.computed(() => {
              const flattenedSlots = flattenVNodes(slots.default?.() || []);
              const slotOptions = extractListSlotOptions(flattenedSlots);
              return mergeSlotOptions(props.options, slotOptions);
          });
          __expose({
              vTableInstance: vue.computed(() => baseTableRef.value?.vTableInstance || null),
          });
          return (_ctx, _cache) => {
              return (vue.openBlock(), vue.createElementBlock(vue.Fragment, null, [
                  vue.createVNode(_sfc_main$3, vue.mergeProps({
                      type: "list",
                      options: computedOptions.value,
                      records: _ctx.records,
                      width: _ctx.width,
                      height: _ctx.height,
                      ref_key: "baseTableRef",
                      ref: baseTableRef
                  }, _ctx.$attrs), null, 16, ["options", "records", "width", "height"]),
                  vue.renderSlot(_ctx.$slots, "default")
              ], 64));
          };
      }
  });

  var _sfc_main$1 = vue.defineComponent({
      __name: 'pivot-table',
      props: {
          options: { type: Object, required: true },
          records: { type: Array, required: false },
          width: { type: [String, Number], required: false },
          height: { type: [String, Number], required: false }
      },
      setup(__props, { expose: __expose }) {
          const props = __props;
          const baseTableRef = vue.shallowRef(null);
          const slots = vue.useSlots();
          const computedOptions = vue.computed(() => {
              const flattenedSlots = flattenVNodes(slots.default?.() || []);
              const slotOptions = extractPivotSlotOptions(flattenedSlots);
              return mergeSlotOptions(props.options, slotOptions);
          });
          __expose({ vTableInstance: vue.computed(() => baseTableRef.value?.vTableInstance || null) });
          return (_ctx, _cache) => {
              return (vue.openBlock(), vue.createElementBlock(vue.Fragment, null, [
                  vue.createVNode(_sfc_main$3, vue.mergeProps({
                      type: "pivot",
                      options: computedOptions.value,
                      records: _ctx.records,
                      width: _ctx.width,
                      height: _ctx.height,
                      ref_key: "baseTableRef",
                      ref: baseTableRef
                  }, _ctx.$attrs), null, 16, ["options", "records", "width", "height"]),
                  vue.renderSlot(_ctx.$slots, "default")
              ], 64));
          };
      }
  });

  var _sfc_main = vue.defineComponent({
      __name: 'pivot-chart',
      props: {
          options: { type: Object, required: true },
          records: { type: Array, required: false },
          width: { type: [String, Number], required: false },
          height: { type: [String, Number], required: false }
      },
      setup(__props, { expose: __expose }) {
          const props = __props;
          const baseTableRef = vue.shallowRef(null);
          const slots = vue.useSlots();
          const computedOptions = vue.computed(() => {
              const flattenedSlots = flattenVNodes(slots.default?.() || []);
              const slotOptions = extractPivotSlotOptions(flattenedSlots);
              return mergeSlotOptions(props.options, slotOptions);
          });
          __expose({ vTableInstance: vue.computed(() => baseTableRef.value?.vTableInstance || null) });
          return (_ctx, _cache) => {
              return (vue.openBlock(), vue.createElementBlock(vue.Fragment, null, [
                  vue.createVNode(_sfc_main$3, vue.mergeProps({
                      type: "chart",
                      options: computedOptions.value,
                      records: _ctx.records,
                      width: _ctx.width,
                      height: _ctx.height,
                      ref_key: "baseTableRef",
                      ref: baseTableRef
                  }, _ctx.$attrs), null, 16, ["options", "records", "width", "height"]),
                  vue.renderSlot(_ctx.$slots, "default")
              ], 64));
          };
      }
  });

  const registerChartModule = (name, chart) => {
      VTable__namespace.register.chartModule(name, chart);
  };

  function ListColumn(props) {
      return null;
  }
  ListColumn.symbol = 'ListColumn';

  function PivotColumnDimension(props) {
      return null;
  }
  PivotColumnDimension.symbol = 'PivotColumnDimension';

  function PivotRowDimension(props) {
      return null;
  }
  PivotRowDimension.symbol = 'PivotRowDimension';

  function PivotColumnHeaderTitle(props) {
      return null;
  }
  PivotColumnHeaderTitle.symbol = 'PivotColumnHeaderTitle';

  function PivotRowHeaderTitle(props) {
      return null;
  }
  PivotRowHeaderTitle.symbol = 'PivotRowHeaderTitle';

  function PivotIndicator(props) {
      return null;
  }
  PivotIndicator.symbol = 'PivotIndicator';

  function PivotCorner(props) {
      return null;
  }
  PivotCorner.symbol = 'PivotCorner';

  function Menu(props) {
      return null;
  }
  Menu.symbol = 'Menu';

  function Tooltip(props) {
      return null;
  }
  Tooltip.symbol = 'Tooltip';

  function Group() {
      return null;
  }
  Group.symbol = 'Group';

  function Image() {
      return null;
  }
  Image.symbol = 'Image';

  function Text() {
      return null;
  }
  Text.symbol = 'Text';

  function Tag(props) {
      return null;
  }
  Tag.symbol = 'Tag';

  function Radio(props) {
      return null;
  }
  Radio.symbol = 'Radio';

  function CheckBox(props) {
      return null;
  }
  CheckBox.symbol = 'CheckBox';

  const version = "1.14.4-alpha.1";

  exports.VTable = VTable__namespace;
  Object.defineProperty(exports, 'register', {
    enumerable: true,
    get: function () { return VTable.register; }
  });
  exports.CheckBox = CheckBox;
  exports.Group = Group;
  exports.Image = Image;
  exports.ListColumn = ListColumn;
  exports.ListTable = _sfc_main$2;
  exports.Menu = Menu;
  exports.PivotChart = _sfc_main;
  exports.PivotColumnDimension = PivotColumnDimension;
  exports.PivotColumnHeaderTitle = PivotColumnHeaderTitle;
  exports.PivotCorner = PivotCorner;
  exports.PivotIndicator = PivotIndicator;
  exports.PivotRowDimension = PivotRowDimension;
  exports.PivotRowHeaderTitle = PivotRowHeaderTitle;
  exports.PivotTable = _sfc_main$1;
  exports.Radio = Radio;
  exports.Tag = Tag;
  exports.Text = Text;
  exports.Tooltip = Tooltip;
  exports.registerChartModule = registerChartModule;
  exports.version = version;

}));
